@@ -17,7 +17,7 @@ from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor
 from django.utils.functional import lazy
 from django.utils.translation import get_language, ugettext
 from parler import signals
-from parler.fields import TranslatedField, LanguageCodeDescriptor
+from parler.fields import TranslatedField, LanguageCodeDescriptor, TranslatedFieldDescriptor
 from parler.managers import TranslatableManager
 from parler.utils.i18n import normalize_language_code, get_language_settings, get_language_title
 import sys
@@ -27,9 +27,11 @@ logger = logging.getLogger(__name__)
 
 
 
-class TranslationDoesNotExist(object):
+class TranslationDoesNotExist(AttributeError):
     """
     A tagging interface to detect missing translations.
+    The exception inherits from :class:`AttributeError` to reflect what is actually happening.
+    It also causes the templates to handle the missing attributes silently, which is very useful in the admin for example.
     """
     pass
 
@@ -74,7 +76,6 @@ def create_translations_model(shared_model, related_name, meta, **fields):
 
     # Create and return the new model
     translations_model = TranslatedFieldsModelBase(name, (TranslatedFieldsModel,), attrs)
-    translations_model.DoesNotExist = type('DoesNotExist', (TranslationDoesNotExist, shared_model.DoesNotExist, translations_model.DoesNotExist,), {})
 
     # Register it as a global in the shared model's module.
     # This is needed so that Translation model instances, and objects which refer to them, can be properly pickled and unpickled.
@@ -138,7 +139,7 @@ class TranslatableModel(models.Model):
         translated_kwargs = {}
         current_language = None
         if kwargs:
-            current_language = kwargs.get('_current_language', None)
+            current_language = kwargs.pop('_current_language', None)
             for field in self._translations_model.get_translated_fields():
                 try:
                     translated_kwargs[field] = kwargs.pop(field)
@@ -157,11 +158,6 @@ class TranslatableModel(models.Model):
             for field, value in translated_kwargs.iteritems():
                 setattr(translation, field, value)
 
-            # Check if other translations were also modified
-            for code, obj in self._translations_cache.iteritems():
-                if code != translation.language_code and obj.is_modified:
-                    obj.save()
-
 
     def get_current_language(self):
         """
@@ -182,6 +178,15 @@ class TranslatableModel(models.Model):
             self._get_translated_model(use_fallback=False, auto_create=True)
 
 
+    def get_fallback_language(self):
+        """
+        Return the fallback language code,
+        which is used in case there is no translation for the currently active language.
+        """
+        lang_dict = get_language_settings(self._current_language)
+        return lang_dict['fallback'] if lang_dict['fallback'] != self._current_language else None
+
+
     def has_translation(self, language_code=None):
         """
         Return whether a translation for the given language exists.
@@ -273,18 +278,67 @@ class TranslatableModel(models.Model):
         ))
 
 
+    def _get_any_translated_model(self):
+        """
+        Return any available translation.
+        Returns None if there are no translations at all.
+        """
+        if self._translations_cache:
+            # There is already a language available in the case. No need for queries.
+            # Give consistent answers if they exist.
+            try:
+                return self._translations_cache.get(self._current_language, None) \
+                    or self._translations_cache.get(self.get_fallback_language(), None) \
+                    or next(t for t in self._translations_cache.itervalues() if t if not None)  # Skip fallback markers.
+            except StopIteration:
+                pass
+
+        try:
+            translation = self._translations_model.objects.filter(master=self)[0]
+        except IndexError:
+            return None
+        else:
+            self._translations_cache[translation.language_code] = translation
+            return translation
+
+
     def save(self, *args, **kwargs):
         super(TranslatableModel, self).save(*args, **kwargs)
         self.save_translations()
 
 
     def save_translations(self):
-        # Also save translated strings.
-        translations = self._get_translated_model()
-        if translations.is_modified:
-            if not translations.master_id:  # Might not exist during first construction
-                translations.master = self
-            translations.save()
+        # Save all translated objects which were fetched.
+        # This also supports switching languages several times, and save everything in the end.
+        for translation in self._translations_cache.itervalues():
+            if translation.is_modified:
+                if not translation.master_id:  # Might not exist during first construction
+                    translation.master = self
+                translation.save()
+
+
+    def safe_translation_getter(self, field, default=None, any_language=False):
+        """
+        Fetch a translated property, and return a default value
+        when both the translation and fallback language are missing.
+
+        When ``any_language=True`` is used, the function also looks
+        into other languages to find a suitable value. This feature can be useful
+        for "title" attributes for example, to make sure there is at least something being displayed.
+        Also consider using ``field = TranslatedField(any_language=True)`` in the model itself,
+        to make this behavior the default for the given field.
+        """
+        try:
+            return getattr(self, field)
+        except TranslationDoesNotExist:
+            pass
+
+        if any_language:
+            translation = self._get_any_translated_model()
+            if translation is not None:
+                return getattr(translation, field, default)
+
+        return default
 
 
 class TranslatedFieldsModelBase(ModelBase):
@@ -362,25 +416,25 @@ class TranslatedFieldsModel(models.Model):
     def shared_model(self):
         return self.__class__.master.field.rel.to
 
-    def save_base(self, raw=False, using=None, update_fields=None, **kwargs):
+    def save_base(self, raw=False, using=None, **kwargs):
         # Send the pre_save signal
         using = using or router.db_for_write(self.__class__, instance=self)
         record_exists = self.pk is not None  # Ignoring force_insert/force_update for now.
-        if self._meta.auto_created:
+        if not self._meta.auto_created:
             signals.pre_translation_save.send(
                 sender=self.shared_model, instance=self,
-                raw=raw, using=using, update_fields=update_fields
+                raw=raw, using=using
             )
 
         # Perform save
-        super(TranslatedFieldsModel, self).save_base(raw=raw, using=using, update_fields=update_fields, **kwargs)
+        super(TranslatedFieldsModel, self).save_base(raw=raw, using=using, **kwargs)
         self._original_values = self._get_field_values()
 
         # Send the post_save signal
         if not self._meta.auto_created:
             signals.post_translation_save.send(
                 sender=self.shared_model, instance=self, created=(not record_exists),
-                update_fields=update_fields, raw=raw, using=using
+                raw=raw, using=using
             )
 
     def delete(self, using=None):
@@ -401,11 +455,8 @@ class TranslatedFieldsModel(models.Model):
 
     @classmethod
     def get_translated_fields(self):
-        fields = self._meta.get_all_field_names()
-        fields.remove('language_code')
-        fields.remove('master')
-        fields.remove('id')   # exists with deferred objects that .only() queries create.
-        return fields
+        # Not using get `get_all_field_names()` because that also invokes a model scan.
+        return [f.name for f, _ in self._meta.get_fields_with_model() if f.name not in ('language_code', 'master', 'id')]
 
     @classmethod
     def contribute_translations(cls, shared_model):
@@ -426,9 +477,14 @@ class TranslatedFieldsModel(models.Model):
                 # Add the proxy field for the shared field.
                 TranslatedField().contribute_to_class(shared_model, name)
             else:
-                if not isinstance(field, models.Field) or field.model is not cls:
+                if not isinstance(field, (models.Field, TranslatedFieldDescriptor)):
                     raise TypeError("The model '{0}' already has a field named '{1}'".format(shared_model.__name__, name))
 
+        # Make sure the DoesNotExist error can be detected als shared_model.DoesNotExist too,
+        # and by inheriting from AttributeError it makes sure (admin) templates can handle the missing attribute.
+        cls.DoesNotExist = type('DoesNotExist', (TranslationDoesNotExist, shared_model.DoesNotExist, cls.DoesNotExist,), {})
+
+
     def __unicode__(self):
         return get_language_title(self.language_code)
 
